/* ------------------------------------------------------------------------------------------
 * Datei:
 *   schloss.js
 * Beschreibung:
 *   Diese Datei ist das Herzstck des Spiels. Sie enthlt das eigentliche
 *   Spiel komplett in ECMAScript.
 * ------------------------------------------------------------------------------------------
 * This work is licensed under the Creative Commons Attribution 3.0 Unported License.
 * To view a copy of this license, visit http://creativecommons.org/licenses/by/3.0/ or send a letter to
 * Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
 * ------------------------------------------------------------------------------------------
 * Copyright (C) 2003-2011 Kai Hildebrandt
 * ------------------------------------------------------------------------------------------
 * Bitmap (fr die Karte):
 *
 * Raum (fest)			Gegenstnde (beweglich)
 *    1 - Nord			    1 - Fackel
 *    2 - Ost				    2 - Speer
 *    4 - Sd				    4 - Handschuh
 *    8 - West			    8 - Krone
 *   16 - Hoch			   16 - Schlssel
 *   32 - Runter			   32 - Knopf
 *   64 - Fallgrube			   64 - Buch
 *  128 - Wache			  128 - Truhe
 *  256 - Wachen			  256 - Messer
 *  512 - Drache			  512 - Zaubertrank
 * 1024 - Zauberer			 1024 - Schild
 *					 2048 - Kobold
 *					 4096 - Tarnkappe
 *					 8192 - Zettel
 *					16384 - Gift
 *
 * Liste der Funktionen:
 *
 * - Action(Raum,GGS):
 *			Wird vor jedem Bildaufbau ausgefhrt
 *			Parameter: Raumdaten, Gegenstnde
 *			Kmmert sich z.B um die Zhler und Meldungen
 *
 * - Ausblenden(id,callResetKommando):
 *                    Blendet das DIV mit der entsprechenden id ein
 *                    und ruft, falls gewuenscht, die Funktion
 *                    ResetKommando() auf.
 *
 * - BildAufbauen():	Bild aufbauen
 *
 * - CheckBit(Mask,Bit,ID):
 *			Hilfsfunktion von BildAufbauen()
 *			Parameter: Maske,Prfbit und ID des Gegenstands/Raumteils
 *			Bei gesetztem Bit wird GGS/Raumteil sichtbar, sonst unsichtbar
 *
 * - CookieCheck():	Hilfsfunktion, die von SpielstandLaden() und SpielstandSpeichern()
 *			bentigt wird. Prft, ob Cookies gesetzt werden drfen und gibt
 *			wahr (1) oder falsch (0) zurck.
 *
 * - Einblenden(id,focusElement):
 *                    Blendet das DIV mit der entsprechenden id aus
 *                    und setzt den Focus auf das angegebene Element.
 *
 * - EingabefeldAktivieren():
 *			Aktiviert Eingabefeld (Kommando) und den Knopf
 *
 * - EingabefeldDeaktivieren():
 *			Deaktiviert Eingabefeld (Kommando) und den Knopf
 *
 * - EndeAnzeigen(ID):
 *			Zeigt das Ende mit variabler Meldung an
 *
 * - FeuerEffekt():	Wird aufgerufen, wenn Drache in Bild ist
 *
 * - FeuerHelper():
 *			Wird von FeuerEffekt() benutzt
 *
 * - Init():		Initialisierung der Variablen
 *
 * - InventurEinblenden():
 *			Die Inventur einblenden
 *
 * - MausHelper():	Animierte Maus, nachdem Zauberer geschrumpft ist
 *			Wird von VerwandlungHelper() aufgerufen
 *
 * - MeldungAnzeigen(Meldung):
 *			Eine Meldung anzeigen
 *
 * - MeldungAusblenden():
 *			Die Meldung ausblenden
 *
 * - Parse():		Kommandoauswertung
 *
 * - PlanAusblenden():
 *			Blendet den Plan nach einem Timeout aus
 *
 * - ResetKommando():	Eingabefeld lschen, Fokus auf Eingabefeld
 *
 * - SpielstandLaden():
 *			Ldt einen gespeicherten Spielstand unter Verwendung von Cookies
 *
 * - SpielstandSpeichern():
 *			Speichert einen gespeicherten Spielstand unter Verwendung von Cookies
 *
 * - Verwandlung():	Die Verwandlung des Zauberers
 *
 * - VerwandlungHelper():
 *			Hilfsfunktion von Verwandlung(). Lsst den Zauberer schrumpfen.
 *
 * - ZuDunkel():	Ausgabe einer Meldung, dass es zu dunkel ist
 */

function Action(Raum,GGS)
{
  // alert("Position: "+x+","+y+","+z);

  /* Der Kerker */
  if (z < 0) {
    EndeAnzeigen(0);
    return;
  }

  /* Das Dach */
  if (z >= zmax) {
    EndeAnzeigen(1);
    return;
  }

  /* Der Wassergraben */
  if (x < 0 || y < 0) {
    if (z == 1) {
      EndeAnzeigen(3);
      return;
    } else {
      EndeAnzeigen(2);
      return;
    }
  }

  /* Die Tr */
  if (y >= ymax) {
    /* Erster Stock */
    if (z == 1) {
      EndeAnzeigen(3);
      return;
    } else {
      /* Schloss verlassen */
      if (TuerOffen == 1) {
        if ((Inv & 8) == 8) {

          /* FIXME: Schloss lst sich in Rauch auf */

	  EndeAnzeigen(16);
	  return;
        } else {
          EndeAnzeigen(4);
          return;
        }
      } else {
	y = ymax - 1;
	BildAufbauen();
        MeldungAnzeigen("Sie sind gegen die Tr gelaufen");
	ResetKommando();
	return;
      }
    }
  }

  /* Die allseits beliebte Fallgrube :-)) */
  if ((Raum & 64) == 64) {
    EndeAnzeigen(5);
    return;
  }

  /* Das Gift */
  if ((Inv & 16384) == 16384) {
    if ( !((Inv & 4) == 4) ) {
      EndeAnzeigen(11);
      return;
    }
  }

  /* Helligkeitszhler erhhen */
  if (KeineZaehler == 0 && HellZaehler <= DunkelAb) HellZaehler++;

  if ( ((Inv & 1) == 0) && ((GGS & 1) == 0) ) {
    if (HellZaehler > DunkelAb) {
      MeldungAnzeigen("Es ist dunkel");
      IstDunkel=1;
    } else {
      if (HellZaehler > (DunkelAb-2)) {
        MeldungAnzeigen("Es wird dunkel");
      }
    }
  } else {
    IstDunkel=0;
  }

  /* keine weiteren Aktionen ausfuehren */
  if (KeineZaehler == 1) {
    KeineZaehler = 0;
    return;
  }

  /* der bse Kobold ;-) */
  if (((Inv & 2) == 2) && (GGS & 2048) == 2048) {
      MeldungAnzeigen("Der Kobold hat ihnen ihren Speer gestohlen");
      AnzahlGGS--;
      Inv = Inv & ~2;   /* ~2 ist binr NICHT 2 */
  }

  /* Tarnkappe */
  if (Tarnkappe == 0) {

  /* Wchter */
  if ((Raum & 128) == 128 || (Raum & 256) == 256) {

    /* Die zwei sich gegenseitig erschlagenden Vollidioten (die Wchter) */
    if (((Raum & 256) == 256) && ((GGS & 128) == 128)) {
      MeldungAnzeigen("Die Wchter haben sich um das Gold gestritten und sich dabei erschlagen");
      PlanRaum[ x + y*xmax + z*xmax*ymax ] = PlanRaum[ x + y*xmax + z*xmax*ymax ] & ~256;
      Wache=0;
      WacheZaehler=0;
    } else {
      Wache=1;   /* keine Truhe im Bild bzw. nur eine Wache */
    }
  }

  if (Wache == 1) { WacheZaehler++; }

  if (WacheZaehler >= 2) {
    EndeAnzeigen(6);
    return;
  }
  /* Ende Wchter */

  } else {
    MeldungAnzeigen("Die Tarnkappe hat sich in Luft aufgelst")
    Tarnkappe=0;
    AnzahlGGS--;
    Inv = Inv & ~4096;
    Wache=0;
    WacheZaehler=0;
  }
  /* Ende Tarnkappe  */

  /* Drache */
  if ((Raum & 512) == 512) {
    Drache=1;
    if ((GGS & 16384) == 16384) {
       MeldungAnzeigen("Der Drache hat das Gift gefressen und sich in Luft aufgelst");
       PlanRaum[ x + y*xmax + z*xmax*ymax ] = PlanRaum[ x + y*xmax + z*xmax*ymax ] & ~512;
       PlanGGS[ x + y*xmax + z*xmax*ymax ] = PlanGGS[ x + y*xmax + z*xmax*ymax ] & ~16384;
       Drache=0;
       DracheZaehler=0;
    } else {
      FeuerEffekt(); if (IstEnde == 1) return;
    }
  }

  if (Drache == 1) { DracheZaehler++; }

  if (DracheZaehler >= 2) {
    EndeAnzeigen(8);
    clearTimeout(Timeout);
    return;
  }
  /* Ende Drache */

  /* Das Buch */
  if ((Inv & 64) == 64) {
    if ((Inv & 1) == 1) {
      MeldungAnzeigen("Die Fackel hat das Buch verbrannt");
      Inv = Inv & ~64;
      AnzahlGGS--;
    } else {
      BuchZaehler++;
    }
  }

  if (BuchZaehler > 4) {
    EndeAnzeigen(10);
    return;
  }
  /* Ende Buch */
}

function Ausblenden(id,callResetKommando)
{
  if (document.getElementById(id)) {
    document.getElementById(id).style.visibility='hidden';
  }

  if (callResetKommando == true) {
    ResetKommando();
  }
}

function BildAufbauen()
{
  /* Bild und Textfenster ausblenden */
  MeldungAusblenden();
  document.getElementById("dBlank").style.visibility="visible";

  /* Daten fr den momentanen Raum holen */
  if (z < 0 || y < 0 || x < 0 || z >= zmax || y >= ymax || x >= xmax) {
    /* Ungltige Position im Spielfeld
     *
     * Wird durch Action(Raum,GGS) abgefangen
     *
     */
    // alert("Ungltige Position:"+x+","+y+","+z);
    var Raum = 0;
    var GGS  = 0;
  } else {
    var Raum = PlanRaum[ x + y*xmax + z*xmax*ymax ];
    var GGS  = PlanGGS[ x + y*xmax + z*xmax*ymax ];
  }

  /*   Nur das Bild Aufbauen, keine Aktionen
   *   (wird fr fortsetzen eines gespeicherten Spielstands
   *    bentigt)
   */

  /* Aktionen ausfhren */
  Action(Raum,GGS); if (IstEnde == 1) return;

  /* Eventuelle nderungen abfangen */
  Raum = PlanRaum[ x + y*xmax + z*xmax*ymax ];
  GGS  = PlanGGS[ x + y*xmax + z*xmax*ymax ];

  /* Diesen Teil auslassen, wenn es dunkel ist */
  if (IstDunkel == 0) {

    /* Sichtbar machen der momentan sichtbaren Dinge
     * bzw. unsichtbar machen des Rests */

    CheckBit(Raum,1,"dTuerNorden");
    CheckBit(Raum,2,"dTuerOsten");
    /* 4 - Tr nach Sden ist nicht sichtbar */
    CheckBit(Raum,8,"dTuerWesten");
    CheckBit(Raum,16,"dLeiterHoch");
    CheckBit(Raum,32,"dLeiterRunter");
    /* 64 - Die Fallgrube */
    CheckBit(Raum,128,"dWache");
    CheckBit(Raum,256,"dWachen");
    CheckBit(Raum,512,"dDrache");
    /* 1024 - der Zauberer (17.05.2003)
     * Fix fr Bug, dass Zauberer verwandelt wurde bevor ein geladener Spielstand
     * fortgesetzt wird. Er ist dann nicht sichtbar (nur die Krone).
     */
    if ((Raum & 1024) == 1024) {
      document.getElementById("dZauberer").style.visibility = "visible";
      document.getElementById("iZauberer").src = "images/zauberer."+ Dateiendung;
      document.getElementById("dMaus").style.visibility = "hidden";
    } else {
      document.getElementById("dZauberer").style.visibility = "hidden";
    }

    /* Gegenstnde */
    CheckBit(GGS,1,"dFackel");
    CheckBit(GGS,2,"dSpeer");
    CheckBit(GGS,4,"dHandschuh");
    CheckBit(GGS,8,"dKrone");
    CheckBit(GGS,16,"dSchluessel");
    CheckBit(GGS,32,"dKnopf");
    CheckBit(GGS,64,"dBuch");
    CheckBit(GGS,128,"dTruhe");
    CheckBit(GGS,256,"dMesser");
    CheckBit(GGS,512,"dZaubertrank");
    CheckBit(GGS,1024,"dSchild");
    CheckBit(GGS,2048,"dKobold");
    CheckBit(GGS,4096,"dTarnkappe");
    CheckBit(GGS,8192,"dZettel");
    CheckBit(GGS,16384,"dGift");

  } /* Ende helligkeitsabhngiger Teil */

  /* Liste der sich im Raum befindenden Gegenstnde anzeigen */
  var Gegenstaende="";
  for (i=0; i < NameGGS.length; i++) {
    if ((GGS & Math.pow(2,i)) == Math.pow(2,i)) {
      Gegenstaende = Gegenstaende.concat(NameGGS[i]+" ");
    }
  }
  document.fAusgabe.iGegenstaende.value = Gegenstaende;

  /* Mgliche Richtungen bestimmen */
  var Richtungen="";
  if ((Raum &  1) ==  1) Richtungen = Richtungen.concat("-N");
  if ((Raum &  2) ==  2) Richtungen = Richtungen.concat("-O");
  if ((Raum &  4) ==  4) Richtungen = Richtungen.concat("-S");
  if ((Raum &  8) ==  8) Richtungen = Richtungen.concat("-W");
  if ((Raum & 16) == 16) Richtungen = Richtungen.concat("-H");
  if ((Raum & 32) == 32) Richtungen = Richtungen.concat("-R");

  /* Richtungen einblenden */
  Richtungen = Richtungen.slice(1,Richtungen.length);   /* Erstes - lschen */
  document.fAusgabe.iRichtungen.value = "Mgliche Richtungen: "+Richtungen;

  /* Fertig, Bild einblenden */
  if (IstDunkel == 0) {
    document.getElementById("dBlank").style.visibility="hidden";
  }
}

function CheckBit(Mask,Bit,ID)
{
  if ((Mask & Bit) == Bit) {
    document.getElementById(ID).style.visibility = "visible";
  } else {
    document.getElementById(ID).style.visibility = "hidden";
  }
}

function CookieCheck()
{
  return (navigator.cookieEnabled == true ? 1 : 0);
}

function Einblenden(id,focusElement)
{
  if (document.getElementById(id)) {
    document.getElementById(id).style.visibility='visible';

    if (document.getElementById(focusElement)) {
      document.getElementById(focusElement).focus();
    }
  }
}

function EingabefeldAktivieren()
{
  document.forms.fKommando.iEingabe.disabled = false;
  document.forms.fKommando.iKnopf.disabled = false;
}

function EingabefeldDeaktivieren()
{
  document.forms.fKommando.iEingabe.blur();
  document.forms.fKommando.iEingabe.disabled = true;
  document.forms.fKommando.iKnopf.disabled = true;
}

function EndeAnzeigen(ID)
{
  var max = 17;

  /* Alle Meldungen ausblenden */
  for (i=0; i < max; i++) {
    document.getElementById("pEnde"+i).style.visibility = "hidden";
  }

  /* Die spezielle Meldung wieder einblenden */
  document.getElementById("pEnde"+ID).style.visibility = "visible";
  document.getElementById("dEnde").style.visibility = "visible";

  /* Die nchste Zeile funktioniert z.B. unter Opera 6.11 nicht */
  // document.getElementById('aNeustart').focus();
  document.forms.fKommando.iEingabe.blur();

  /* Kommandozeile ausblenden */
  document.forms.fAusgabe.style.visibility = "hidden";
  document.forms.fKommando.style.visibility = "hidden";
  MeldungAusblenden();

  IstEnde=1;   /* Rcksprung-Flag, Abarbeitung weiterer Funktionen beenden */
}

function FeuerEffekt()
{
  /* Bild aufbauen (ziemlich umstndlich und speziell) und einblenden */
  CheckBit(PlanRaum[ x + y*xmax + z*xmax*ymax ],2,"dTuerOsten");
  CheckBit(PlanRaum[ x + y*xmax + z*xmax*ymax ],8,"dTuerWesten");
  CheckBit(PlanRaum[ x + y*xmax + z*xmax*ymax ],512,"dDrache");
  document.getElementById("dBlank").style.visibility="hidden";

  n=0;   /* Zhler, wird erst bei Feuereffekt interessant (der nicht eingebaut ist) */
  FeuerHelper();
}

function FeuerHelper()
{
  n++;

  if (n < 2) {

    /* FIXME: Hier knnte noch ein Feuereffekt stehen */

    MeldungAnzeigen("Der Drache speit Feuer");
    EingabefeldDeaktivieren();
    Timeout = setTimeout("FeuerHelper()",5000);
  } else {
    if ((Inv & 1024) == 0) {
      EndeAnzeigen(7);
      return;
    } else {
      MeldungAusblenden();
      EingabefeldAktivieren();
      ResetKommando();
      return;
    }
  }
}

function Init()
{
  x=1; y=2; z=0;			/* Position im Spiel */

  xmax=4; ymax=4; zmax=2;		/* Planbegrenzungen */

  Inv=0;				/* Bitmap der momentan getragenen Gegestnde */
  AnzahlGGS=0;				/* Anzahl dieser Gegestnde */
  MaxAnzahlGGS=4;			/* Maximale Anzahl der Gegenstnde, die man
  					 * tragen kann */

  Wache=0;				/* Flag, ob man sich mit Wache in einem Raum
  					 * befindet */
  WacheZaehler=0;			/* Zhlt, wie lange man sich mit Wache im Raum
  					 * befindet */

  Drache=0;				/* s.o. fr den Drachen */
  DracheZaehler=0;

  BuchZaehler=0;			/* Zhlt, wie lange man das Buch trgt */
  BuchOffen=0;				/* Flag fr geffnetes Buch */
  BuchMax=5;				/* Maximaler Wert fr das Buch tragen */

  Tarnkappe=0;				/* Flag fr die Benutzung der Tarnkappe */

  HellZaehler=0;			/* Zhlt, wie lange es noch hell ist */
  DunkelAb=9;				/* Ab diesem Wert ist es dunkel */
  IstDunkel=0;				/* Flag, ob dunkel (keine Fackel) */

  TuerOffen=0;				/* Flag fr geffnete Tr */

  IstEnde=0;				/* Zeiger, ob Ende erreicht ist fr
  					 * gezielten Rcksprung */

  KeineZaehler=0;				/* Flag, ob beim Bildaufbau Aktionen ausgefhrt
  					 * werden sollen (siehe SpielstandLaden)
					 */

  Timeout=0;				/* Zwischenspeicher fr Timeouts */

  /* Hier beginnt die Karte
   *
   * die Rume */
  PlanRaum = new Array(   6,   14,   15,   24,
			  64,    5,    3,  140,
			  39,   11,  140,    5,
			   1,   18,  271,   41,

			  64,   10,   12,   36,
			1026,  522,  269,    1,
			   6,  142,   11,   12,
			  17,   33,    4,    1);

  /* die Gegenstnde */
  PlanGGS = new Array(  16384,  512,    0,    1,
			     0,  256,    0,   64,
			     0,    0,  128, 2048,
			     2, 4096, 1024,    0,

			     0, 8192,    0,    0,
			     0,    0,    0,   16,
			     0,    0,    2,    0,
			     0, 2048,    4,   32);

  NameGGS = new Array ("Fackel",
  			"Speer",
			"Handschuh",
			"Krone",
			"Schlssel",
			"Knopf",
			"Buch",
			"Truhe",
			"Messer",
			"Zaubertrank",
			"Schild",
			"Kobold",
			"Tarnkappe",
			"Zettel",
			"Gift");

  /* Den ersten Raum aufbauen */
  BildAufbauen(); if (IstEnde == 1) return;

  ResetKommando();
}

function InventurEinblenden()
{
  /* Opera lsst das ndern von versteckten Formularfeldern durch
   * Skripte nicht zu. Deshalb erst sichtbar machen, dann ndern.
   */
  document.getElementById("dInv").style.visibility = "visible";

  /* Liste lschen */
  for(i=0; i < 4; i++) {
    document.getElementById("iInv"+i).value = "";
  }

  /* Liste aktualisieren */
  var pos=0;   /* Aktuelle Position zum Einschieben */
  for(i=0; i < NameGGS.length; i++) {
    if ((Inv & Math.pow(2,i)) == Math.pow(2,i)) {
      if (pos > 3) {
        alert("Fehler in InventurEinblenden(): pos nimmt verbotenen Wert an");
      } else {
        document.getElementById("iInv"+pos).value = NameGGS[i];
        pos++;
      }
    }
  }
  document.forms.fInventur.iInvKnopf.focus();   /* Focus auf Ausblendeknopf */
}

function MausHelper()
{
  /* Mausposition setzen (das Bild) */
  document.getElementById("dMaus").style.left = xm +"px";
  document.getElementById("dMaus").style.top  = ym +"px";

  ym--;
  xm = xm + 15;

  if (xm < 580) {
    setTimeout("MausHelper()",150);
  } else {
    document.getElementById("dMaus").style.visibility = "hidden";

    /* Zauberer weg, Krone da */
    PlanRaum[ x + y*xmax + z*xmax*ymax ] = PlanRaum[ x + y*xmax + z*xmax*ymax ] & ~1024;
    PlanGGS[ x + y*xmax + z*xmax*ymax ] = PlanGGS[ x + y*xmax + z*xmax*ymax ] | 8;

    /* Eingabefeld aktivieren */
    EingabefeldAktivieren();

    BildAufbauen(); if (IstEnde == 1) return;
    ResetKommando();
  }
}

function MeldungAnzeigen(Meldung)
{
  document.fTextfenster.iMeldung.style.visibility = "visible";
  document.getElementById("dTextfenster").style.visibility = "visible";
  document.fTextfenster.iMeldung.value = Meldung;
}

function MeldungAusblenden()
{
  document.getElementById("dTextfenster").style.visibility = "hidden";
  document.fTextfenster.iMeldung.style.visibility = "hidden";
}

function Parse()
{
  // das eingegebene Kommando in eine schoene Form bringen:
  // - alles in Kleinbuchstaben konvertieren
  // - fuehrende und folgende Spaces entfernen
  // - alle mehrfachen Spaces in einzelne konvertieren
  // - alle kleinen  in ue (wegen dem Schluessel bei nimm und verliere)
  ko = document.fKommando.iEingabe.value.toLowerCase().replace(/^ +/,"").replace(/ +$/,"").replace(/  +/g," ").replace(//g,"ue");

  var nm = 1;   /* Flag, ob "nicht mglich" angezeigt werden soll */

  // alert("Sie gaben ein: " + ko);

  /* keine Eingabe, evtl. vertippt */
  if (ko == "") { ResetKommando(); return; }

  if (ko.slice(0,5) == "debug") {
    // alert("Position: "+x+","+y+","+z);
    // alert("Hellzaehler: "+HellZaehler);
    ResetKommando();
    return;
  }

  /* Daten fr den momentanen Raum holen */
  var Raum = PlanRaum[ x + y*xmax + z*xmax*ymax ];
  var GGS  = PlanGGS[ x + y*xmax + z*xmax*ymax ];

  /* Spielstand */
  if (ko == "laden" || ko == "load") { SpielstandLaden(); ResetKommando(); return; }
  if (ko == "speichern" || ko == "save") { SpielstandSpeichern(); ResetKommando(); return; }

  if (ko == "inv" || ko =="inventur" || ko == "liste" || ko == "list") {
    InventurEinblenden();
    return;
  }

  if (ko.slice(0,4) == "hilf" || ko == "help") {
    Einblenden("dHilfe","iHilfeKnopf");
    return;
  }

  /* der bse Zauberer */
  if ((Raum & 1024) == 1024) {
    if (ko != "simsalakadabra-abrakabum") {
      EndeAnzeigen(9);
      return;
    } else {
      Verwandlung();
      return;
    }
  }

  /* sich bewegen */
  if (ko == "n" && (Raum &  1) ==  1) { y--; nm=0; }
  if (ko == "o" && (Raum &  2) ==  2) { x++; nm=0; }
  if (ko == "s" && (Raum &  4) ==  4) { y++; nm=0; }
  if (ko == "w" && (Raum &  8) ==  8) { x--; nm=0; }
  if (ko == "h" && (Raum & 16) == 16) { z++; nm=0; }
  if (ko == "r" && (Raum & 32) == 32) { z--; nm=0; }

  /* sich selbst in eine Maus verzaubern */
  if (ko.slice(0,3) == "abr" || ko.slice(0,3) == "sim") {
    EndeAnzeigen(15);
    return;
  }

  /* Tarnkappe benutzen */
  if ((ko == "benutze tarnkappe") && ((Inv & 4096) == 4096)) {
    Tarnkappe = 1;
    MeldungAnzeigen("Sie sind jetzt unsichtbar.");
    ResetKommando();
    return;
  }

  /* der bse Kobold ;-) */
  if (ko == "nimm kobold" || ko.slice(0,14) == "verjage kobold") {
    var was = "mitnehmen";
    if (ko.slice(0,7) == "verjage") { was = "verjagen"; }

    /* die Folgen */
    MaxAnzahlGGS = 2;
    PlanGGS[x + y*xmax + z*xmax*ymax] = PlanGGS[x + y*xmax + z*xmax*ymax] | Inv;
    Inv = 0;
    AnzahlGGS = 0;

    /* verlorene Gegenstnde anzeigen */
    BildAufbauen(); if (IstEnde == 1) return;

    MeldungAnzeigen("Der Kobold wollte sich nicht "+ was +" lassen und hat sie in einen Zwerg verzaubert");
    ResetKommando();
    return;
  }

  /* nehmen von Gegenstnden */
  if (ko.slice(0,4) == "nimm") {
    if (AnzahlGGS == MaxAnzahlGGS) {
      MeldungAnzeigen("Sie knnen nicht soviel tragen");
      ResetKommando();
      return;
    } else {
      var t = ko.slice(5,ko.length);
      for (i=0; i < NameGGS.length; i++) {
	if (NameGGS[i].toLowerCase().replace(//g,"ue") == t) {
	  if ((GGS & Math.pow(2,i)) == Math.pow(2,i)) {
	    Inv = Inv | Math.pow(2,i);
	    AnzahlGGS++;
	    PlanGGS[ x + y*xmax + z*xmax*ymax ] = PlanGGS[ x + y*xmax + z*xmax*ymax ] & ~Math.pow(2,i);
	    break;   /* Weiter nach for-Schleife */
	  } else {
	    MeldungAnzeigen("Diesen Gegenstand gibt es hier nicht");
	    ResetKommando();
	    return;
	  }
	}
      }
      if (i == NameGGS.length) {
	MeldungAnzeigen("Diesen Gegenstand gibt es hier nicht");
	ResetKommando();
	return;
      }
      nm = 0;
    }
  }   /* Ende nehmen */

  /* verlieren von Gegenstnden */
  if (ko.slice(0,8) == "verliere") {
    if (AnzahlGGS > 0) {
      var t = ko.slice(9,ko.length);

      /* alle Gegenstaende auf einmal loswerden */
      if (t == "alles") {
        PlanGGS[x + y*xmax + z*xmax*ymax] = PlanGGS[x + y*xmax + z*xmax*ymax] | Inv;
        Inv = 0;
        AnzahlGGS = 0;
        nm = 0;
      } else {
        for (i=0; i < NameGGS.length; i++) {
          if (NameGGS[i].toLowerCase().replace(//g,"ue") == t) {
    	    if ((Inv & Math.pow(2,i)) == Math.pow(2,i)) {
    	      Inv = Inv & ~Math.pow(2,i);
    	      AnzahlGGS--;
    	      PlanGGS[ x + y*xmax + z*xmax*ymax ] = PlanGGS[ x + y*xmax + z*xmax*ymax ] | Math.pow(2,i);
    	      break;   /* Weiter nach for-Schleife */
            }
          }
        }
        nm = 0;
        if (i == NameGGS.length) {
          MeldungAnzeigen("Das haben sie ja gar nicht");
          ResetKommando();
          return;
        }
      }
    } else {
      MeldungAnzeigen("Das haben sie ja gar nicht");
      ResetKommando();
      return;
    }
  }   /* Ende verlieren */

  /* ffnen von Gegenstnden */
  if (ko.slice(0,5) == "ffne" || ko.slice(0,6) == "oeffne") {
    if (ko.slice(0,5) == "ffne") {
      var t = ko.slice(6,ko.length);
    } else {
      var t = ko.slice(7,ko.length);
    }

    /* Buch */
    if (t == "buch") {
      if ((Inv & 64) == 64) {
        BuchOffen = 1;
        Action(Raum,GGS); if (IstEnde == 1) return;
        MeldungAnzeigen("Das Buch ist offen");
        ResetKommando();
        return;
      } else {
        MeldungAnzeigen("Sie besitzen kein Buch");
	ResetKommando();
	return;
      }
    }

    /* Truhe */
    if (t == "truhe") {
      if ((Inv & 128) == 128) {
        MeldungAnzeigen("Die Truhe ist offen. Sie enthlt Gold.");
        ResetKommando();
        return;
      } else {
        MeldungAnzeigen("Sie besitzen keine Truhe");
        ResetKommando();
        return;
      }
    }

    /* Tr */
    if (t == "tr" || t == "tuer") {
      if (x == 2 && y == 3 && z == 0)
      {
        if ((Inv & 16) == 16) {
          TuerOffen = 1;
  	Action(Raum,GGS); if (IstEnde == 1) return;
          MeldungAnzeigen("Die Tr ist offen.");
          ResetKommando();
          return;
        } else {
          MeldungAnzeigen("Die Tr ist verschlossen.");
          ResetKommando();
          return;
        }
      } else {
          MeldungAnzeigen("Da ist keine Tr.");
          ResetKommando();
          return;
      }
    }
  }   /* Ende ffne */

  /* lesen */
  if (ko.slice(0,4) == "lies") {
    var t = ko.slice(5,ko.length);

    /* Buch */
    if (t == "buch") {
      if ((Inv & 64) == 64) {
        if (IstDunkel == 1) {
	  Action(Raum,GGS); if (IstEnde == 1) return;
	  ZuDunkel();
          ResetKommando();
	  return;
        } else {
          if (BuchOffen == 0) {
	    Action(Raum,GGS); if (IstEnde == 1) return;
            MeldungAnzeigen("Das Zauberschloss");
            ResetKommando();
            return;
          } else {
            MeldungAusblenden();
	    Action(Raum,GGS); if (IstEnde == 1) return;
            document.getElementById('dBuchtext').style.visibility = "visible";
            document.getElementById('iBuchKnopf').focus();
	    return;
	  }
	}
      } else {
        MeldungAnzeigen("Sie besitzen kein Buch");
        ResetKommando();
        return;
      }
    }

    /* Zettel */
    if (t == "zettel") {
      if ((Inv & 8192) == 8192) {
        if (IstDunkel == 1) {
	  Action(Raum,GGS); if (IstEnde == 1) return;
	  ZuDunkel();
          ResetKommando();
	  return;
        } else {
	  MeldungAusblenden();
	  Action(Raum,GGS); if (IstEnde == 1) return;
          document.getElementById("dPlan").style.visibility = "visible";
          setTimeout("PlanAusblenden()",5000);

          /* Plan aus Inventur entfernen, damit man ihn nicht verlieren kann */
          Inv = Inv & ~8192;
          AnzahlGGS--;

          ResetKommando();
	  return;
        }
      } else {
        MeldungAnzeigen("Sie besitzen keinen Zettel");
        ResetKommando();
        return;
      }
    }

  } /* Ende lesen */

  /* verjagen */
  if (ko.slice(0,7) == "verjage") {
    var t = new Array();
    t = ko.split(" ");   /* Kommando in einzelne Wrter spalten */
    if (t[1] == "wache" || t[1] == "wachen" || t[1] == "wchter" || t[1] == "waechter") {
      if ((Raum & 128) == 128 || (Raum & 256) == 256) {
        if (t.length < 4 || t[2] != "mit") {
	  MeldungAnzeigen("Womit denn?");
	  ResetKommando();
	  return;
	} else {
	  var waffe = t[3];
	  switch(waffe) {
            case "speer":
              if ((Inv & 2) == 0) {
	          MeldungAnzeigen("Sie besitzen keinen Speer");
	          ResetKommando();
	          return;
	      } else {
	        /* ein zweiter Wchter? */
	        if ((Raum & 256) == 256) {
	          EndeAnzeigen(12);
	          return;
	        } else {
		  Wache = 0;
		  WacheZaehler = 0;
	          PlanRaum[ x + y*xmax + z*xmax*ymax ] = PlanRaum[ x + y*xmax + z*xmax*ymax ] & ~128;
		  nm = 0;
	        }
	      }
	      break;

	    case "messer":
              if ((Inv & 256) == 0) {
	          MeldungAnzeigen("Sie besitzen kein Messer");
	          ResetKommando();
	          return;
	      } else {
	        EndeAnzeigen(13);
	        return;
	      }
	      break;

	    case "gift":
              if ((Inv & 16384) == 0) {
	          MeldungAnzeigen("Sie besitzen kein Gift");
	          ResetKommando();
	          return;
	      } else {
	        EndeAnzeigen(14);
	        return;
	      }
	      break;

	    default:
	          MeldungAnzeigen("Das besitzen sie nicht");
	          ResetKommando();
	          return;
	      break;
	  }
        }
      } else {
        if (t[1] == "wache") {
	  MeldungAnzeigen("Da ist keine Wache");
	} else {
	  MeldungAnzeigen("Da sind keine Wchter");
	}
	setTimeout("MeldungAusblenden()",5000);
	ResetKommando();
	return;
      }
    }
  }   /* Ende verjagen */

  /* drcken */
  if ((ko == "drcke knopf" || ko == "druecke knopf") && ((GGS & 32) == 32)) {
    if (x-1 >= 0) {
      PlanRaum[ x-1 + y*xmax + z*xmax*ymax ] = PlanRaum[ x-1 + y*xmax + z*xmax*ymax ] | 2;
    }
    PlanRaum[ x + y*xmax + z*xmax*ymax ] = PlanRaum[ x + y*xmax + z*xmax*ymax ] | 8;
    PlanGGS[ x + y*xmax + z*xmax*ymax ] = PlanGGS[ x + y*xmax + z*xmax*ymax ] & ~32;
    nm=0;
  }   /* Ende drcken */

  /* trinken */
  if (ko.slice(0,5) == "trink") {
    var t = ko.slice(6,ko.length);
    if ((t == "Gift") && ((Inv & 16384) == 16384)) {
      EndeAnzeigen(11);
      return;
    }
    if ((t == "zaubertrank") && ((Inv & 512) == 512)) {
      Inv = Inv & ~512;
      AnzahlGGS--;
      if (MaxAnzahlGGS == 4) {
        Action(Raum,GGS); if (IstEnde == 1) return;
        MaxAnzahlGGS = 2;
	if (AnzahlGGS > MaxAnzahlGGS) {
          PlanGGS[ x + y*xmax + z*xmax*ymax ] = PlanGGS[ x + y*xmax + z*xmax*ymax ] | Inv;
	  Inv = 0;
	  AnzahlGGS = 0;
	  BildAufbauen();
	}
	MeldungAnzeigen("Sie sind ein Zwerg");
	ResetKommando();
	return;
      } else {
        Action(Raum,GGS); if (IstEnde == 1) return;
        MaxAnzahlGGS = 4;
	MeldungAnzeigen("Sie sind wieder gro");
	ResetKommando();
	return;
      }
    }
  } /* Ende trinken */

  /* nicht mglich */
  if (nm == 1) {
    MeldungAnzeigen("Nicht mglich");
    ResetKommando();
    return;
  }

  /* Bild neu aufbauen */
  BildAufbauen(); if (IstEnde == 1) return;

  ResetKommando();
}

function PlanAusblenden()
{
  document.getElementById("dPlan").style.visibility="hidden";
  MeldungAnzeigen("Der Plan hat sich in Luft aufgelst");
  ResetKommando();
}

function ResetKommando()
{
  if (document.forms.fKommando.iEingabe.disabled == false) {
    document.forms.fKommando.iEingabe.value = "";
    document.forms.fKommando.iEingabe.focus();
  }
}

function SpielstandLaden()
{
  if (CookieCheck() == true) {
    if (document.cookie) {
      var Spielstand = document.cookie;

      /*  "Spielstand=" entfernen */
      Spielstand = Spielstand.slice(11,Spielstand.length);

      /* Daten auseinander pflcken */
      Spielstand = Spielstand.split(",");

      var spos = 0;   /* Zhler fr aktuellen Wert */

      var k = xmax * ymax * zmax;   /* Kartenabmessungen */

      for (i=0; i<k; i++) {
        PlanRaum[i] = parseInt(Spielstand[spos++]);
      }

      for (i=0; i<k; i++) {
        PlanGGS[i] = parseInt(Spielstand[spos++]);
      }

      /* Die aktuelle Position */
      x = parseInt(Spielstand[spos++]);
      y = parseInt(Spielstand[spos++]);
      z = parseInt(Spielstand[spos++]);

      /* Inventur */
      MaxAnzahlGGS = parseInt(Spielstand[spos++]);
      AnzahlGGS = parseInt(Spielstand[spos++]);
      Inv = parseInt(Spielstand[spos++]);

      /* diverse Zhler */
      Wache = parseInt(Spielstand[spos++]);
      WacheZaehler = parseInt(Spielstand[spos++]);
      Drache = parseInt(Spielstand[spos++]);
      DracheZaehler = parseInt(Spielstand[spos++]);
      BuchZaehler = parseInt(Spielstand[spos++]);
      BuchOffen = parseInt(Spielstand[spos++]);
      Tarnkappe = parseInt(Spielstand[spos++]);
      HellZaehler = parseInt(Spielstand[spos++]);
      TuerOffen = parseInt(Spielstand[spos++]);

      KeineZaehler = 1;
      BildAufbauen();
      MeldungAnzeigen("Der Spielstand wurde erfolgreich geladen");
      ResetKommando();
    } else {
      alert ("Es ist kein gespeicherter Spielstand vorhanden");
    }
  } else {
    alert ("Bitte aktivieren sie Cookies");
  }
}

function SpielstandSpeichern()
{
  /*   Lebensdauer des Cookies berechnen
   *   Ein Jahr sollte reichen !?
   */
  var Verfall = new Date();
  var einJahr = Verfall.getTime() + (365 * 24 * 60 * 60 * 1000);
  Verfall.setTime(einJahr);

  if (CookieCheck() == true) {
    /* Cookie erstellen */
    var Spielstand = "Spielstand=";

    var k = xmax * ymax * zmax;

    for(i=0; i<k; i++) {
      Spielstand = Spielstand.concat(PlanRaum[i]+",");
    }

    for(i=0; i<k; i++) {
      Spielstand = Spielstand.concat(PlanGGS[i]+",");
    }

    /* Die aktuelle Position */
    Spielstand = Spielstand.concat(x+","+y+","+z+",");

    /* Inventur */
    Spielstand = Spielstand.concat(MaxAnzahlGGS+",");
    Spielstand = Spielstand.concat(AnzahlGGS+",");
    Spielstand = Spielstand.concat(Inv+",");

    /* diverse Zhler */
    Spielstand = Spielstand.concat(Wache+",");
    Spielstand = Spielstand.concat(WacheZaehler+",");
    Spielstand = Spielstand.concat(Drache+",");
    Spielstand = Spielstand.concat(DracheZaehler+",");
    Spielstand = Spielstand.concat(BuchZaehler+",");
    Spielstand = Spielstand.concat(BuchOffen+",");
    Spielstand = Spielstand.concat(Tarnkappe+",");
    Spielstand = Spielstand.concat(HellZaehler+",");
    Spielstand = Spielstand.concat(TuerOffen);

    /* Verfallsdatum einfgen */
    Spielstand = Spielstand + "; expires=" + Verfall.toGMTString();

    /* Spielstand als Cookie speichern */
    document.cookie = Spielstand;

    MeldungAnzeigen("Der Spielstand wurde erfolgreich gespeichert");
    ResetKommando();
  } else {
    alert ("Bitte aktivieren sie Cookies");
  }
}

function Verwandlung()
{
  nr=1;
  EingabefeldDeaktivieren();
  VerwandlungHelper(nr);
}

function VerwandlungHelper(nr)
{
  if (nr <= 15) {
    document.getElementById("iZauberer").src = "images/verwandlung"+nr+"."+Dateiendung;
    setTimeout("VerwandlungHelper(nr++)",250);
  } else {
      document.getElementById("dMaus").style.visibility = "visible";
      xm = 280; ym = 360;   /* Maus Anfangsposition */
      MausHelper();
      return;
  }
}

function ZuDunkel()
{
  MeldungAnzeigen("Dafr ist es zu dunkel");
  ResetKommando();
}
